@@ -309,12 +309,12 @@ static void SetCyrillicLetters(Translator *tr)
 	tr->letter_bits_offset = OFFSET_CYRILLIC;
 	memset(tr->letter_bits,0,sizeof(tr->letter_bits));
 	SetLetterBits(tr,LETTERGP_A,(char *)ru_vowels);
-	SetLetterBits(tr,1,ru_soft);
-	SetLetterBits(tr,2,(char *)ru_consonants);
-	SetLetterBits(tr,3,ru_hard);
-	SetLetterBits(tr,4,ru_nothard);
-	SetLetterBits(tr,5,ru_voiced);
-	SetLetterBits(tr,6,ru_ivowels);
+	SetLetterBits(tr,LETTERGP_B,ru_soft);
+	SetLetterBits(tr,LETTERGP_C,(char *)ru_consonants);
+	SetLetterBits(tr,LETTERGP_H,ru_hard);
+	SetLetterBits(tr,LETTERGP_F,ru_nothard);
+	SetLetterBits(tr,LETTERGP_G,ru_voiced);
+	SetLetterBits(tr,LETTERGP_Y,ru_ivowels);
 	SetLetterBits(tr,LETTERGP_VOWEL2,(char *)ru_vowels);
 }  // end of SetCyrillicLetters
 
@@ -957,6 +957,9 @@ SetLengthMods(tr,3);  // all equal
 			tr->langopts.stress_rule = 8;   // ?? 1st syllable if it is heavy, else 2nd syllable
 			tr->langopts.param[LOPT_UNPRONOUNCABLE] = 1;   // disable check for unpronouncable words
 			tr->langopts.numbers = NUM_OMIT_1_HUNDRED;
+			tr->langopts.numbers2 = NUM2_MYRIADS;
+			tr->langopts.break_numbers = 0x1111110;
+			tr->langopts.max_digits = 20;
 		}
 		break;
 
@@ -1113,7 +1116,7 @@ SetLengthMods(tr,3);  // all equal
 			tr->langopts.stress_rule = STRESSPOSN_1R;        // stress on final syllable
 			tr->langopts.stress_flags =  0x6 | 0x10 | 0x2000 | 0x20000; 
 			tr->langopts.numbers = NUM_DECIMAL_COMMA | NUM_DFRACTION_2 | NUM_HUNDRED_AND | NUM_AND_UNITS | NUM_ROMAN;
-			tr->langopts.numbers2 = NUM2_MULTIPLE_ORDINAL; 
+			tr->langopts.numbers2 = NUM2_MULTIPLE_ORDINAL | NUM2_NO_TEEN_ORDINALS; 
 			SetLetterVowel(tr,'y');
 			ResetLetterBits(tr,0x2);
 			SetLetterBits(tr,1,"bcdfgjkmnpqstvxz");      // B  hard consonants, excluding h,l,r,w,y
@@ -1356,6 +1359,16 @@ SetLengthMods(tr,3);  // all equal
 		}
 		break;
 
+	case L('t','t'):   // Tatar
+      {
+			SetCyrillicLetters(tr);
+			SetupTranslator(tr,stress_lengths_fr,stress_amps_fr);
+			tr->langopts.stress_rule = STRESSPOSN_1R;      // stress on final syllable
+			tr->langopts.stress_flags = 0x20;  //no automatic secondary stress
+			tr->langopts.numbers = NUM_SINGLE_STRESS | NUM_DECIMAL_COMMA | NUM_OMIT_1_HUNDRED | NUM_OMIT_1_THOUSAND | NUM_DFRACTION_4;
+      }
+      break;
+
 	case L('u','k'):   // Ukrainian
 		{
 			SetCyrillicLetters(tr);
